/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.c
 * Copyright (C) Maurice Clerc 2010 <Maurice.Clerc@WriteMe.com>
 
 Generate a matrix of random rotation in dimension D
 In passing, it can also generate the matrix of the inverse rotation, 
 for it is useful for some applications.
 
 Theoretically perfect, but in practice the precision is not very good.
 Typically, instead of zero, you may have something like 1.e-17
 
 * 
 * Random_rotation is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Random_rotation is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "stdio.h"
#include "math.h"
#include <stdlib.h>
#define ulong unsigned long // To generate pseudo-random numbers with KISS
#define RAND_MAX_KISS ((unsigned long) 4294967295)


#define DMax 10
#define randOption 0  // 0 => RNG Kiss, 1 => RNG of C
#define zero 1.e-15

// Structures

struct matrixSquare  {int size; double m[DMax][DMax];} ;
struct vector 
{  
	int size;				// Number of dimensions D  
	double v[DMax]; 		// Components
};
struct pairs
{
	int size;
	int p[DMax*(DMax-1)/2][2];
};
// Global variables
double pi;
double twoPi;
double theta[DMax*(DMax-1)/2];

// Subroutines
double alea (double a, double b) ;
double alea_normal (double mean, double std_dev) ;
struct vector alea_sphere(int D, double radius1,double radius2,
							int randType, double randGamma);
							
							
void displayM(struct matrixSquare M);
struct matrixSquare matrixProduct(struct matrixSquare M1, struct matrixSquare M2);
struct matrixSquare randomRot(int D,int option);
struct pairs pairsListe(int D);
void	seed_rand_kiss(ulong seed); 
ulong	rand_kiss(); // For the pseudo-random number generator KISS

double distance(double p1[],double p2[],int D);


FILE * f_run;

int main()
{
	
	int D;
	int i;
	struct matrixSquare M;
	struct matrixSquare M1; // Just for test.
	int t,test;
	
	int pMax=10000;// Just to check the distribution
	double points[pMax][DMax]; 
	int neigh[pMax];
	double dist;
	int t1;
	double mean;
	double std;
	int min,max;
	
	pi=acos(-1);
	twoPi=2*pi;
	
	f_run = fopen ("f_run.txt", "w");  
	
	D=10;
	
	M=randomRot(D,1);
	printf("\n Rotation");
	displayM(M);
	M1=randomRot(D,-1); // Should be M^(-1)
	
	printf("\n Inverse rotation");
	displayM(M1);
	
	M=matrixProduct(M1,M);
	printf("\n Should be the unit matrix");
	displayM(M);

	
/*
	// TEST. Generate points that should be random _on_ the unit sphere
	// according to an uniform distribution
	D=10;
	test=10000;
	
	// Rotate the point (1,0,0,...0) "test" times, at random
	for(t=0;t<test;t++)
	{
		M=randomRot(D,1);
		for(i=0;i<D;i++) points[t][i]=M.m[0][i];
		// Save on a file
		for(i=0;i<D;i++) fprintf(f_run,"%f ",M.m[0][i]); fprintf(f_run,"\n");
	}


	// Check the norms. Should be 1
	for(t=0;t<test;t++)
	{
		dist=0;
		for(i=0;i<D;i++) dist=dist+points[t][i]*points[t][i];
		dist=sqrt(dist);
		if(fabs(dist-1)>zero)
			printf("\nt = %i norm = %f",t,dist);
	}
	

	// Check the uniformity
	// For each t, neight[t] should be more or less the same
	dist=0.4; // Distance threshold. Need to be tuned so that the mean is about D
	for(t=0;t<test;t++)
	{
		neigh[t]=0;
		for(t1=0;t1<test;t1++)
		{
			if(t1==t) continue;
			if(distance(points[t],points[t1],D)<=dist) neigh[t]++;	
		}
	fprintf(f_run,"%i",neigh[t]); fprintf(f_run,"\n");
	}

	mean=0;
	max=0;
	min=test;
	for(t=0;t<test;t++) 
	{   
		mean=mean+neigh[t];
		if(neigh[t]<min) min=neigh[t];
		if(neigh[t]>max) max=neigh[t];
	}
	mean=mean/test;
	
	std=0;
	for(t=0;t<test;t++) std=std+pow(neigh[t]-mean,2);
	std=sqrt(std/test);
	printf("\nDimension %i",D);
	printf("\n Distance treshold %f",dist);
	printf("\n Neighbours:  min %i, max %i",min,max);
	printf("\n Mean and std deviation over %i points: %f (%f)",test,mean,std);
*/	
	return (0);
}
//==============================================================================
void displayM(struct matrixSquare M)
{
	int option=1; // 0 => as it is. 1 => small values set to zero
	int i,j;
	double zz;
	if(option==1) printf("\n  Warning: values smaller than %e are set to 0",zero);
	for(i=0;i<M.size;i++)
	{
		printf("\n");
		for(j=0;j<M.size;j++) 
		{   
			zz=M.m[i][j];
			if(option==1 && fabs(zz)<zero) zz=0;
			printf("%e ",zz);
		}
	}
}
//==============================================================================
struct matrixSquare randomRot(int D,int option)
{
	// Build the matrix of random rotation
	// We save the different random angles theta in order to be able to also
	// build the inverse matrix
	struct matrixSquare M,Mr;
	M.size=D; Mr.size=D;
	
	int d1,d2;
	int i,j;
	int r;
	int rank;
	struct vector v;
	struct pairs pairs;
	double zz;

	
	// Generate all pairs of axes
	pairs=pairsListe(D);
 //for(i=0;i<pairs.size;i++) printf("\n %i %i",pairs.p[i][0],pairs.p[i][1]);	
	//-------------
	for(i=0;i<D;i++) for(j=0;j<D;j++) M.m[i][j]=0; for(i=0;i<D;i++) M.m[i][i]=1;
	
	if(option==1) 
	{
		zz=alea(0,twoPi); theta[0]=zz;
		d1=0;d2=1;
	}
	else 
	{
		zz=-theta[pairs.size-1];
		d1=pairs.p[pairs.size-1][0];
		d2=pairs.p[pairs.size-1][1];		
	}
	
	M.m[d1][d1]=cos(zz);  M.m[d1][d2]=-sin(zz);
	M.m[d2][d1]=sin(zz);  M.m[d2][d2]=cos(zz);

	for(rank=1;rank<pairs.size;rank++)
	{
		for(i=0;i<D;i++) for(j=0;j<D;j++) Mr.m[i][j]=0; for(i=0;i<D;i++) Mr.m[i][i]=1;
		if(option==1)
		{
			zz=alea(0,twoPi);
			theta[rank]=zz;
			r=rank;
		}
		else
		{
			r=pairs.size-rank-1;
			zz=-theta[r];
		}
		
		d1=pairs.p[r][0];
		d2=pairs.p[r][1];
		Mr.m[d1][d1]=cos(zz);  Mr.m[d1][d2]=-sin(zz);
		Mr.m[d2][d1]=sin(zz);  Mr.m[d2][d2]=cos(zz);
		
		M=matrixProduct(M,Mr);
	}

	return M;
}
//==============================================================================
struct matrixSquare matrixProduct(struct matrixSquare M1, struct matrixSquare M2)
{
	int i,j,k;
	struct matrixSquare M;
	M.size=M1.size;
	double zz;

	for(i=0;i<M.size;i++)
	{	
		for(j=0;j<M.size;j++)
		{
			zz=0;
			for(k=0;k<M.size;k++)
			{
				zz=zz+M1.m[i][k]*M2.m[k][j];
			}
			M.m[i][j]=zz;
		}
	}

	return M;
}
//==============================================================================
struct pairs pairsListe(int D)
{
	int i;
	struct pairs liste;
	int rank;

	if(D==2)
	{
		liste.p[0][0]=0;
		liste.p[0][1]=1;
		liste.size=1;
		return liste;
	}

	liste=pairsListe(D-1);
	rank=liste.size;
	for(i=0;i<D-1;i++)
	{
		liste.p[rank][0]=i;
		liste.p[rank][1]=D-1;
		rank=rank+1;
	}
	liste.size=rank;
	return liste;
}
//==============================================================================
double distance(double p1[],double p2[],int D)
{
	int d;
	double dist=0;
	
	for(d=0;d<D;d++)
	{
		dist=dist+pow(p1[d]-p2[d],2);	
	}
	dist=sqrt(dist);
	return dist;

}
//==============================================================================
#include "alea.c"